import greenfoot.*;
import java.util.*;

/**
 * The Animated class is the superclass for all Actors that need to be
 * animated. It includes methods for adding "skits"(little animations
 * performed over a time period) removing skits, activating skits,
 * retreving skits based on ID, and animating every act cycle.
 * 
 * @author TriMil
 * @version 1.0
 */
public abstract class Animated extends Actor
{
    /**
     * A list of skits
     */
    private ArrayList<Skit> skits = new ArrayList<Skit>(3);
    
    /**
     * The current cycle
     */
    private int cycle;
    
    /**
     * The current skit
     */
    private Skit currentSkit;
    
    /**
     * Must be called at the beginning of act() method.
     * 
     * This method checks if any of the scenes should start, starts 
     * any that do need to, and then increments the cycle. If no skit
     * is running, most of this metod will not get run.
     */
    public void run()
    {
        if(currentSkit == null){return;}
        for(Scene s: currentSkit.getScript())
        {
            if(s.getStart() == cycle)
            {
                if(s.getImage() == null)
                {
                    currentSkit = null;
                    cycle = 0;
                    return;
                }
                setImage(s.getImage());
            }
        }
        cycle++;
    }
    
    /**
     * Adds a new Skit to the list.
     * 
     * @param newSkit The new Skit being added to the list.
     */
    protected void addSkit(Skit newSkit)
    {
        skits.add(newSkit);
    }
    
    /**
     * Gets a Skit object from its ID.
     * 
     * @param id The ID of the Skit being added
     * 
     * @return The skit with the specified ID. 
     * If an invalid ID is specified, null will be returned
     */
    protected Skit getSkit(String id)
    {
        for(Skit s: skits)
        {
            if(s.getID().equals(id))
            {
                return s;
            }
        }
        return null;
    }
    
    /**
     * Removes a Skit from the list
     * 
     * @param id The ID of the Skit being removed;
     * 
     * @return A true or false value indicating if removal was succesful.
     * Returns true if the Skit was removed, returns false if the ID was
     * invalid or if an exception occured.
     */
    protected boolean removeSkit(String id)
    {
        Skit s = getSkit(id);
        if(s == null) return false;
        
        return skits.remove(s);
    }
    
    /**
     * Starts the specified Skit. No Skit will start if the ID is invalid.
     * 
     * @param id The ID of the Skit to be started.
     */
    protected void startSkit(String id)
    {
        currentSkit = getSkit(id);
        cycle = 0;
    }
}








